@@ -1014,7 +1014,7 @@ fts_table_fetch_doc_ids(
 	}
 
 	if (alloc_bk_trx) {
-		trx_free(trx);
+		trx->free();
 	}
 
 	return(error);
@@ -1716,7 +1716,8 @@ fts_optimize_free(
 	mem_heap_t*	heap = static_cast<mem_heap_t*>(optim->self_heap->arg);
 
 	trx_commit_for_mysql(optim->trx);
-	trx_free(optim->trx);
+	optim->trx->free();
+	optim->trx = NULL;
 
 	fts_doc_ids_free(optim->to_delete);
 	fts_optimize_graph_free(&optim->graph);
@@ -2388,7 +2389,7 @@ fts_optimize_table_bk(
 	dict_table_t*	table = slot->table;
 	dberr_t		error;
 
-	if (fil_table_accessible(table)
+	if (table->is_accessible()
 	    && table->fts && table->fts->cache
 	    && table->fts->cache->deleted >= FTS_OPTIMIZE_THRESHOLD) {
 		error = fts_optimize_table(table);
@@ -2589,6 +2590,11 @@ fts_optimize_remove_table(
 	if (fts_opt_start_shutdown) {
 		ib::info() << "Try to remove table " << table->name
 			<< " after FTS optimize thread exiting.";
+		/* If the table can't be removed then wait till
+		fts optimize thread shuts down */
+		while (fts_optimize_wq) {
+			os_thread_sleep(10000);
+		}
 		return;
 	}
 
@@ -2621,9 +2627,13 @@ fts_optimize_remove_table(
 
 	os_event_destroy(event);
 
-	ut_d(mutex_enter(&fts_optimize_wq->mutex));
-	ut_ad(!table->fts->in_queue);
-	ut_d(mutex_exit(&fts_optimize_wq->mutex));
+#ifdef UNIV_DEBUG
+	if (!fts_opt_start_shutdown) {
+		mutex_enter(&fts_optimize_wq->mutex);
+		ut_ad(!table->fts->in_queue);
+		mutex_exit(&fts_optimize_wq->mutex);
+	}
+#endif /* UNIV_DEBUG */
 }
 
 /** Send sync fts cache for the table.
@@ -2789,8 +2799,7 @@ static void fts_optimize_sync_table(dict_table_t *table,
   if (!sync_table)
     return;
 
-  if (sync_table->fts && sync_table->fts->cache &&
-      fil_table_accessible(sync_table))
+  if (sync_table->fts && sync_table->fts->cache && sync_table->is_accessible())
   {
     fts_sync_table(sync_table, false);
     if (process_message)
