@@ -2,7 +2,7 @@
 
 Copyright (c) 1995, 2016, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2008, Google Inc.
-Copyright (c) 2017, 2019, MariaDB Corporation.
+Copyright (c) 2017, 2020, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -36,6 +36,7 @@ Created 9/11/1995 Heikki Tuuri
 
 #include "os0event.h"
 #include "ut0mutex.h"
+#include "ilist.h"
 
 /** Counters for RW locks. */
 struct rw_lock_stats_t {
@@ -105,9 +106,7 @@ struct rw_lock_t;
 struct rw_lock_debug_t;
 #endif /* UNIV_DEBUG */
 
-typedef UT_LIST_BASE_NODE_T(rw_lock_t)	rw_lock_list_t;
-
-extern rw_lock_list_t			rw_lock_list;
+extern ilist<rw_lock_t> rw_lock_list;
 extern ib_mutex_t			rw_lock_list_mutex;
 
 /** Counters for RW locks. */
@@ -320,8 +319,7 @@ rw_lock_validate(
 	const rw_lock_t*	lock);	/*!< in: rw-lock */
 #endif /* UNIV_DEBUG */
 /******************************************************************//**
-Low-level function which tries to lock an rw-lock in s-mode. Performs no
-spinning.
+Low-level function which tries to lock an rw-lock in s-mode.
 @return TRUE if success */
 UNIV_INLINE
 ibool
@@ -563,26 +561,23 @@ readers, a writer may queue for x-lock by decrementing lock_word: no
 new readers will be let in while the thread waits for readers to
 exit. */
 
-struct rw_lock_t
+struct rw_lock_t :
 #ifdef UNIV_DEBUG
-	: public latch_t
+	public latch_t,
 #endif /* UNIV_DEBUG */
+	public ilist_node<>
 {
-	/** Holds the state of the lock. */
-	std::atomic<int32_t>	lock_word;
+  ut_d(bool created= false;)
 
-	/** 1: there are waiters */
-	std::atomic<int32_t>	waiters;
+  /** Holds the state of the lock. */
+  Atomic_relaxed<int32_t> lock_word;
+
+  /** 0=no waiters, 1=waiters for X or SX lock exist */
+  Atomic_relaxed<uint32_t> waiters;
 
 	/** number of granted SX locks. */
 	volatile ulint	sx_recursive;
 
-	/** This is TRUE if the writer field is RW_LOCK_X_WAIT; this field
-	is located far from the memory update hotspot fields which are at
-	the start of this struct, thus we can peek this field without
-	causing much memory bus traffic */
-	bool		writer_is_wait_ex;
-
 	/** The value is typically set to thread id of a writer thread making
 	normal rw_locks recursive. In case of asynchronous IO, when a non-zero
 	value of 'pass' is passed then we keep the lock non-recursive.
@@ -616,9 +611,6 @@ struct rw_lock_t
 	/** Count of os_waits. May not be accurate */
 	uint32_t	count_os_wait;
 
-	/** All allocated rw locks are put into a list */
-	UT_LIST_NODE_T(rw_lock_t) list;
-
 #ifdef UNIV_PFS_RWLOCK
 	/** The instrumentation hook */
 	struct PSI_rwlock*	pfs_psi;
