@@ -199,7 +199,7 @@ enum ESphRankMode
 	SPH_RANK_PROXIMITY_BM25		= 0,	///< default mode, phrase proximity major factor and BM25 minor one
 	SPH_RANK_BM25				= 1,	///< statistical mode, BM25 ranking only (faster but worse quality)
 	SPH_RANK_NONE				= 2,	///< no ranking, all matches get a weight of 1
-	SPH_RANK_WORDCOUNT			= 3,	///< simple word-count weighting, rank is a weighted sum of per-field keyword occurence counts
+	SPH_RANK_WORDCOUNT			= 3,	///< simple word-count weighting, rank is a weighted sum of per-field keyword occurrence counts
 	SPH_RANK_PROXIMITY			= 4,	///< phrase proximity
 	SPH_RANK_MATCHANY			= 5,	///< emulate old match-any weighting
 	SPH_RANK_FIELDMASK			= 6,	///< sets bits where there were matches
@@ -596,6 +596,7 @@ private:
 
 	struct Override_t
 	{
+		Override_t() : m_dIds(PSI_INSTRUMENT_MEM), m_dValues(PSI_INSTRUMENT_MEM) {}
 		union Value_t
 		{
 			uint32		m_uValue;
@@ -695,7 +696,7 @@ handlerton sphinx_hton =
 	NULL,	// create_cursor_read_view
 	NULL,	// set_cursor_read_view
 	NULL,	// close_cursor_read_view
-	HTON_CAN_RECREATE
+	HTON_CAN_RECREATE | HTON_AUTOMATIC_DELETE_TABLE
 };
 #else
 static handlerton * sphinx_hton_ptr = NULL;
@@ -737,17 +738,18 @@ static int sphinx_init_func ( void * p )
 	{
 		sphinx_init = 1;
 		void ( pthread_mutex_init ( &sphinx_mutex, MY_MUTEX_INIT_FAST ) );
-		sphinx_hash_init ( &sphinx_open_tables, system_charset_info, 32, 0, 0,
-			sphinx_get_key, 0, 0 );
+                sphinx_hash_init ( PSI_NOT_INSTRUMENTED, &sphinx_open_tables,
+                                   system_charset_info, 32, 0, 0,
+                                   sphinx_get_key, 0, 0 );
 
 		#if MYSQL_VERSION_ID > 50100
 		handlerton * hton = (handlerton*) p;
-		hton->state = SHOW_OPTION_YES;
 		hton->db_type = DB_TYPE_AUTOASSIGN;
 		hton->create = sphinx_create_handler;
 		hton->close_connection = sphinx_close_connection;
 		hton->show_status = sphinx_show_status;
 		hton->panic = sphinx_panic;
+		hton->drop_table= [](handlerton *, const char*) { return -1; };
 		hton->flags = HTON_CAN_RECREATE;
 		#endif
 	}
@@ -769,10 +771,8 @@ static int sphinx_close_connection ( handlerton * hton, THD * thd )
 {
 	// deallocate common handler data
 	SPH_ENTER_FUNC();
-	void ** tmp = thd_ha_data ( thd, hton );
-	CSphTLS * pTls = (CSphTLS *) (*tmp);
+	CSphTLS * pTls = (CSphTLS *) thd_get_ha_data ( thd, hton );
 	SafeDelete ( pTls );
-	*tmp = NULL;
 	SPH_RET(0);
 }
 
@@ -844,7 +844,7 @@ bool sphinx_show_status ( THD * thd )
 
 #if MYSQL_VERSION_ID>50100
 	// 5.1.x style stats
-	CSphTLS * pTls = (CSphTLS*) ( *thd_ha_data ( thd, hton ) );
+	CSphTLS * pTls = (CSphTLS*) ( thd_get_ha_data ( thd, hton ) );
 
 #define LOC_STATS(_key,_keylen,_val,_vallen) \
 	stat_print ( thd, sphinx_hton_name, strlen(sphinx_hton_name), _key, _keylen, _val, _vallen );
@@ -1306,6 +1306,7 @@ CSphSEQuery::CSphSEQuery ( const char * sQuery, int iLength, const char * sIndex
 	, m_fGeoLongitude ( 0.0f )
 	, m_sComment ( (char*) "" )
 	, m_sSelect ( (char*) "*" )
+        , m_dOverrides (PSI_INSTRUMENT_MEM)
 
 	, m_pBuf ( NULL )
 	, m_pCur ( NULL )
@@ -2118,11 +2119,7 @@ int ha_sphinx::open ( const char * name, int, uint )
 
 	thr_lock_data_init ( &m_pShare->m_tLock, &m_tLock, NULL );
 
-	#if MYSQL_VERSION_ID>50100
-	*thd_ha_data ( table->in_use, ht ) = NULL;
-	#else
-	table->in_use->ha_data [ sphinx_hton.slot ] = NULL;
-	#endif
+	thd_set_ha_data ( table->in_use, ht, 0 );
 
 	SPH_RET(0);
 }
@@ -2805,23 +2802,16 @@ CSphSEThreadTable * ha_sphinx::GetTls()
 {
 	SPH_ENTER_METHOD()
 	// where do we store that pointer in today's version?
-	CSphTLS ** ppTls;
-#if MYSQL_VERSION_ID>50100
-	ppTls = (CSphTLS**) thd_ha_data ( table->in_use, ht );
-#else
-	ppTls = (CSphTLS**) &current_thd->ha_data[sphinx_hton.slot];
-#endif // >50100
+	CSphTLS * pTls = (CSphTLS*) thd_get_ha_data ( table->in_use, ht );
 
 	CSphSEThreadTable * pTable = NULL;
 	// allocate if needed
-	if ( !*ppTls )
+	if ( !pTls )
 	{
-		*ppTls = new CSphTLS ( this );
-		pTable = (*ppTls)->m_pHeadTable;
-	} else
-	{
-		pTable = (*ppTls)->m_pHeadTable;
+		pTls = new CSphTLS ( this );
+		thd_set_ha_data(table->in_use, ht, pTls);
 	}
+	pTable = pTls->m_pHeadTable;
 
 	while ( pTable && pTable->m_pHandler!=this )
 		pTable = pTable->m_pTableNext;
@@ -2829,8 +2819,8 @@ CSphSEThreadTable * ha_sphinx::GetTls()
 	if ( !pTable )
 	{
 		pTable = new CSphSEThreadTable ( this );
-		pTable->m_pTableNext = (*ppTls)->m_pHeadTable;
-		(*ppTls)->m_pHeadTable = pTable;
+		pTable->m_pTableNext = pTls->m_pHeadTable;
+		pTls->m_pHeadTable = pTable;
 	}
 
 	// errors will be handled by caller
@@ -3052,7 +3042,7 @@ int ha_sphinx::get_rec ( byte * buf, const byte *, uint )
 	}
 
 	#if MYSQL_VERSION_ID>50100
-	MY_BITMAP * org_bitmap = dbug_tmp_use_all_columns ( table, &table->write_set );
+	my_bitmap_map * org_bitmap = dbug_tmp_use_all_columns ( table, table->write_set );
 	#endif
 	Field ** field = table->field;
 
@@ -3198,7 +3188,7 @@ int ha_sphinx::get_rec ( byte * buf, const byte *, uint )
 	m_iCurrentPos++;
 
 	#if MYSQL_VERSION_ID > 50100
-	dbug_tmp_restore_column_map ( &table->write_set, org_bitmap );
+	dbug_tmp_restore_column_map ( table->write_set, org_bitmap );
 	#endif
 
 	SPH_RET(0);
@@ -3371,7 +3361,7 @@ int ha_sphinx::rename_table ( const char *, const char * )
 // if start_key matches any rows.
 //
 // Called from opt_range.cc by check_quick_keys().
-ha_rows ha_sphinx::records_in_range ( uint, key_range *, key_range * )
+ha_rows ha_sphinx::records_in_range ( uint, const key_range *, const key_range *, page_range *)
 {
 	SPH_ENTER_METHOD();
 	SPH_RET(3); // low number to force index usage
@@ -3532,7 +3522,7 @@ CSphSEStats * sphinx_get_stats ( THD * thd, SHOW_VAR * out )
 #if MYSQL_VERSION_ID>50100
 	if ( sphinx_hton_ptr )
 	{
-		CSphTLS * pTls = (CSphTLS *) *thd_ha_data ( thd, sphinx_hton_ptr );
+		CSphTLS * pTls = (CSphTLS *) thd_get_ha_data ( thd, sphinx_hton_ptr );
 
 		if ( pTls && pTls->m_pHeadTable && pTls->m_pHeadTable->m_bStats )
 			return &pTls->m_pHeadTable->m_tStats;
@@ -3597,7 +3587,7 @@ int sphinx_showfunc_words ( THD * thd, SHOW_VAR * out, char * sBuffer )
 #if MYSQL_VERSION_ID>50100
 	if ( sphinx_hton_ptr )
 	{
-		CSphTLS * pTls = (CSphTLS *) *thd_ha_data ( thd, sphinx_hton_ptr );
+		CSphTLS * pTls = (CSphTLS *) thd_get_ha_data ( thd, sphinx_hton_ptr );
 #else
 	{
 		CSphTLS * pTls = (CSphTLS *) thd->ha_data[sphinx_hton.slot];
